import os
import hashlib

def generate_hash_function(file_name, algorithm='sha256'):
    try:
        with open(file_name, 'rb') as file:
            hasher = hashlib.new(algorithm)
            for chunk in iter(lambda: file.read(4096), b''):
                hasher.update(chunk)
            return hasher.hexdigest()
    except FileNotFoundError:
        print("File not found.")
        return None
    except PermissionError:
        print("No permissions to read the file")
        return None

def final_hash_function(data):
    return hashlib.sha256(data.encode()).hexdigest()

# Indication of evaluation tool file/module names (execution file and library files)
# Note: if evaluation tool files/modules are in other directories, indicate proper paths to them
current_directory = os.getcwd()
module_name = os.path.join(current_directory, 'build','odu','bin', 'ric_stub', 'odu_fuzzer')
lib1_name = os.path.join(current_directory, 'build','odu','bin', 'ric_stub', 'libasn_common.a')
lib2_name = os.path.join(current_directory, 'build','odu','bin', 'ric_stub', 'libcm.a')
lib3_name = os.path.join(current_directory, 'build','odu','bin', 'ric_stub', 'libric.a')
lib4_name = os.path.join(current_directory, 'build','odu','bin', 'ric_stub', 'libasn_e2ap.a')
lib5_name = os.path.join(current_directory, 'build','odu','bin', 'ric_stub', 'libmt.a')

# Indication of Evaluation test results file/files
results_file = os.path.join(current_directory, 'test_results', 'test_results.zip')

# Indication of Evaluation report file
report_file = os.path.join(current_directory, 'test_results', 'PUT_evaluation_report-v1.0.pdf')

# Indication of Product Under Test (PUT)
# Note: if identification of PUT is implemented as hash calculation of PUT file/files, set flag PUT_HASH to 1 and indicate 
# proper path; otherwhise set flag PUT_HASH to 0 and provide PUT identification string 
# (PUT hash provided by developer, PUT name and version etc.)
PUT_HASH = 0

if PUT_HASH == 1:
	# when identification of PUT is implemented as hash calculation of PUT file/files
	put_identificator = os.path.join(current_directory, 'put_files', 'put.zip')
else:
	# when identification of PUT is provided as PUT name and version, or hash received from developer
	put_identificator = "PUT_name_and_version"
	#put_identificator = "cbc1c425ff2b1a96dd42627efd84b9b8cec8fa8c27efadd9d2082c03c8f34978"


if PUT_HASH == 1:
	component_list = [module_name, lib1_name, lib2_name, lib3_name, lib4_name, lib5_name, results_file, report_file, put_identificator] 
else:
	component_list = [module_name, lib1_name, lib2_name, lib3_name, lib4_name, lib5_name, results_file, report_file] 

# Hash creation for each component
hashes = [generate_hash_function(hash_component) for hash_component in component_list]

# Concatenation of hashes
concat_hash = ''.join(hashes)
if PUT_HASH == 0:
	concat_hash = concat_hash + put_identificator

# Calculation of final hash for an evaluation
result_hash = final_hash_function(concat_hash)

print("Hashes of the components:")
for hash_component, hash in zip(component_list, hashes):
    print(f"{hash_component}: {hash}")

print("\nFinal Evaluation Hash:")
print(result_hash)


